/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd.
 * lightdm-kiran-greeter is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 *
 * Author:     liuxinhao <liuxinhao@kylinos.com.cn>
 */
#include "useravatarwidget.h"
#include <QFile>
#include <QPainter>
#include <qt5-log-i.h>

#define DEFAULT_USER_AVATAR ":/images/user_180.png"

UserAvatarWidget::UserAvatarWidget(QWidget *parent) : QWidget(parent)
{
    setDefaultImage();
}

void UserAvatarWidget::setImage(const QString &path)
{
    QFile file(path);
    if (!file.exists())
    {
        KLOG_WARNING() << "UserAvatar: file path[" << path << "] is no't exist";
    }
    if (!file.isReadable())
    {
        KLOG_WARNING() << "UserAvatar: file path[" << path << "] can't read";
    }
    if (m_pixmap.load(path))
    {
        KLOG_WARNING() << "UserAvatar: load file " << path << "successed";
        m_scaledPixmap = scalePixmapAdjustSize(m_pixmap);
    }
    else
    {
        KLOG_WARNING() << "UserAvatar: file path[" << path << "] load failed.";
        if (path != DEFAULT_USER_AVATAR)
        {
            setDefaultImage();
        }
    }
    update();
}

void UserAvatarWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    QPen     pen;
    if (!isVisible())
    {
        return;
    }
    painter.setRenderHints(
        QPainter::SmoothPixmapTransform | QPainter::HighQualityAntialiasing | QPainter::Antialiasing);
    double radius = (this->width() < this->height() ? this->width() : this->height()) / 2;
    if (!m_scaledPixmap.isNull())
    {
        QPainterPath painterPath;
        painterPath.addEllipse((this->width() - (radius * 2)) / 2, (this->height() - (radius * 2)) / 2, radius * 2,
                               radius * 2);
        painter.setClipPath(painterPath);
        painter.drawPixmap((this->width() - m_scaledPixmap.width()) / 2, (this->height() - m_scaledPixmap.height()) / 2,
                           m_scaledPixmap);
    }
    QWidget::paintEvent(event);
}

void UserAvatarWidget::resizeEvent(QResizeEvent *event)
{
    if ((!m_pixmap.isNull()) && (!m_scaledPixmap.isNull()) && (m_scaledPixmap.size() != this->size()))
    {
        m_scaledPixmap = scalePixmapAdjustSize(m_pixmap);
    }
    QWidget::resizeEvent(event);
}

QPixmap UserAvatarWidget::scalePixmapAdjustSize(const QPixmap &pixmap)
{
    double radius = (this->width() < this->height() ? this->width() : this->height()) / 2;
    //NOTE:拉升保持长宽比，尽可能放大，不留白
    QPixmap temp = pixmap.scaled(radius * 2, radius * 2, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
    return temp;
}

void UserAvatarWidget::setDefaultImage()
{
    setImage(DEFAULT_USER_AVATAR);
}
